use strict;

=head1 NAME

Net::ITE - OOP-ish interface to the Internet Topic Exchange

=head1 SYNOPSIS

 use Net::ITE;

 my $ite = Net::ITE->new();

 my $topic = $ite->topic("montreal_quebec_canada");
 my $posts = $topic->posts();

 while (my $item = $posts->next()) {
    print $item->title()."\n";
    print $item->excerpt()."\n";
 }

 my $res = $topic->ping({title=>"foo",
			url=>"http://foo.com/123",
			excerpt=>"yadda yadda yadda"});

=head1 DESCRIPTION

OOP-ish interface to the Internet Topic Exchange.

=head1 NOTES

=over 4

=item *

This is alpha quality software. Buyer beware.

=item *

The error handling sucks and will be addressed in future releases.

=back

=cut

=head1 Net::ITE

=cut

package Net::ITE;

$Net::ITE::VERSION = '0.01';

=head2 __PACKAGE__->new($blogname)

Returns an object.Woot!

=cut

sub new {
  my $pkg  = shift;
  my $blog = shift;
  return bless \$blog,$pkg;
}

=head2 $ite->topics()

Returns an I<Iterator> object of I<Topic> objects.

I<Doesn't work yet.>

=cut

sub topics {
  my $self = shift;

  warn "I can't do this yet\n";
  return undef;

  my $data = Network->get(Constants->TOPICS);

  if (! $data) {
    warn "Unable to topics.\n";
    return undef;
  }

  my $items = Parser->parse_feed($data);

  if (! $items) {
    warn "Unable to parse topics.\n";
    return undef;
  }

  map { $_ = {title=>$_->{title},blog=>$self} } @$items;

  return Iterator->new("Topic",$items);
}

=head2 $ite->topic($topic)

Returns a I<Topic> object.

=cut

sub topic {
  my $self = shift;
  return Topic->new({title=>$_[0],blog=>$self});
}

=head1 Topic

=cut

package Topic;

my $tb = undef;

sub new {
  my $pkg  = shift;
  return bless $_[0],$pkg;
}

=head2 $topic->title()

=cut

sub title { return shift->{title}; }

=head2 $topic->url()

=cut

sub url {
  my $self = shift;
  return Constants->TOPIC.$self->title()."/";
}

=head2 $topic->ping(\%args)

=over 4

=item *

B<blog_name>

If this property is not passed then the value passed to the I<Net::ITE>
constructor will be used.

=item *

B<title>

=item *

B<url>

=item *

B<excerpt>

=back

=cut

sub ping {
  my $self = shift;
  my $data = shift;

  $data->{ping_url} = $self->url();
  $data->{blog_name} ||= ${$self->{blog}};

  if (! $tb) {
    require XML::TrackBack;
    $tb = XML::TrackBack->new();
  }

  return $tb->send_ping($data);
}

=head2 $topic->posts()

Returns an I<Iterator> object of I<Post> objects.

=cut

sub posts {
  my $self = shift;

  my $data = Network->get($self->url()."rss");

  if (! $data) {
    warn "Unable to retrieve posts for topic.\n";
    return undef;
  }

  my $posts = Parser->parse_feed($data);

  if (! $posts) {
    warn "Unable to parse posts for topic.\n";
    return undef;
  }

  return Iterator->new("Post",$posts);
}

=head1 Post

=cut

package Post;

sub new {
  my $pkg  = shift;
  return bless $_[0],$pkg;
}

=head2 $post->title()

=cut

sub title   { return shift->{title} }

=head2 $post->url()

=cut

sub url     { return shift->{link}; }

=head2 $pkg->excerpt()

=cut

sub excerpt { return shift->{description}; }

=head1 Iterator

=cut

package Iterator;

sub new {
  my $pkg = shift;
  return bless {pkg=>$_[0],data=>$_[1],count=>0}, $pkg;
}

=head2 $it->count()

=cut

sub count { 
  my $self = shift;
  return scalar @{$self->{data}};
}

=head2 $it->next()

Returns an object.Woot!

=cut 

sub next {
  my $self = shift;
  if (my $data = $self->{data}->[$self->{count}++]) {
    return $self->{pkg}->new($data);
  }
}

package Network;
my $ua = undef;

sub get {
  my $pkg = shift;

  if (! $ua) {
    require LWP::UserAgent;
    require HTTP::Request;
    $ua = LWP::UserAgent->new();
  }

  my $res = $ua->request(HTTP::Request->new(GET=>$_[0]));

  if (! $res->is_success()) {
    warn "Failed to retrieve data, ".$res->message()."\n";
    return undef;
  }

  return $res->content_ref();
}

package Parser;
use XML::RSS;

my $rss = undef;

sub parse_feed {
  my $pkg    = shift;
  my $sr_rss = shift;

  $rss ||= XML::RSS->new();

  eval { $rss->parse($$sr_rss); };
  if ($@) { 
    warn $@,"\n";
    return undef;
  }

  return $rss->{items};
}

package Constants;

use constant ITE    => "http://topicexchange.com";
use constant TOPICS => undef;
use constant TOPIC  => ITE."/t/";

return 1;

=head1 VERSION

0.01

=head1 DATE

January 14, 2003

=head1 AUTHOR

Aaron Straup Cope

=head1 REQUIREMENTS

=over 4

=item *

I<LWP::UserAgent>

=item *

I<HTTP::Request>

=item *

I<XML::RSS>

=item *

I<XML::TrackBack>

=back

=head1 SEE ALSO

http://topicexchange.com

http://www.mplode.com/tima/files/TrackBack.pm

=head1 LICENSE

Copyright (c) 2003 Aaron Straup Cope, All Rights Reserved.

This is free software, you may use it and distribute it under the same terms as Perl itself.

=cut

